import ts from "typescript";
import {CompilerCodeCompiler} from "../compiler-code";

export interface TransformCodeResult {
    state: Record<any, any>;
    lifeCycles: Record<any, any>;
    methods: Record<any, any>;
}
/**
 * AST 分析代码转换 结构化的 json，  此处顺带会将 ts tsx 代码全部转化为 es5
 *
 *
 * example:
 * class LowcodeComponent extends Component {
 *   state = {
 *     "text": "outer",
 *     "isShowDialog": false
 *   }
 *
 *   componentDidMount() {
 *     console.log('did mount');
 *   }
 *
 *   componentWillUnmount() {
 *     console.log('will unmount');
 *   }
 *
 *   testFunc() {
 *     console.log('test func');
 *   }
 *
 *   onClick() {
 *     this.setState({
 *       isShowDialog: true
 *     });
 *   }
 * }
 *
 * 通过结构话化的转换为：
 * {
 *     "state": {
 *         "\"text\"": {
 *             "type": "JSExpression",
 *             "value": "\"odduter\""
 *         },
 *         "\"isShowDialog\"": {
 *             "type": "JSExpression",
 *             "value": "false"
 *         }
 *     },
 *     "lifeCycles": {
 *         "componentDidMount": {
 *             "type": "JSFunction",
 *             "value": "{\n    console.log('did mount');\n}\n"
 *         },
 *         "componentWillUnmount": {
 *             "type": "JSFunction",
 *             "value": "{\n    console.log('will unmount');\n}\n"
 *         }
 *     },
 *     "methods": {
 *         "testFunc": {
 *             "type": "JSFunction",
 *             "value": "{\n    console.log('test func');\n}\n"
 *         },
 *         "onClick": {
 *             "type": "JSFunction",
 *             "value": "{\n    this.setState({\n        isShowDialog: true\n    });\n}\n"
 *         }
 *     }
 * }
 * @param code
 */
export function TransformCodeToStructuringJson(code: string): TransformCodeResult {
    // 结果对象
    const result: TransformCodeResult = {
        state: {},
        lifeCycles: {},
        methods: {}
    };
    const sourceFile = ts.createSourceFile(
        "example.ts",
        code,
        ts.ScriptTarget.Latest,
        true
    );
    // React 生命周期方法枚举
    enum ReactLifeCycle {
        componentDidMount = "componentDidMount",
        componentWillUnmount = "componentWillUnmount",
        // 其他 React 生命周期方法可以在这里添加
    }
    function visit(node: ts.Node) {
        if (ts.isPropertyDeclaration(node) && ts.isObjectLiteralExpression(node.initializer!)) {
            const properties: { type?: string; value?: any}[] = node.initializer.properties.map((prop) => {
                const value = (prop as any).initializer.getText();
                //const js = CompilerCodeCompiler(value);
                return {
                    [(prop as any).name.getText()]: {
                        type: "JSExpression",
                        value: value,
                        source: value
                    }
                };
            })

            result.state = properties.reduce((acc, curr) => ({...acc, ...curr}), {});
        }

        if (ts.isMethodDeclaration(node)) {
            const functionName = node.name.getText();
            let functionBody = node.getText() || "";
            functionBody = `function ${functionBody}`
            const js = CompilerCodeCompiler(functionBody);
            if (functionName in ReactLifeCycle) {
                result.lifeCycles[functionName] = {
                    type: "JSFunction",
                    value: js,
                    source: functionBody,
                    name: functionName
                };
            } else {
                result.methods[functionName] = {
                    type: "JSFunction",
                    value: js,
                    source: functionBody,
                    name: functionName
                };
            }
        }

        ts.forEachChild(node, visit);
    }
    ts.forEachChild(sourceFile, visit);
    return result;
}

/**
 * // 假设您已经有组件的 state 和其他必要的数据
 * const componentState = {
 *   isShowDialog: false
 * };
 *
 * // 解析 JSON 对象中的方法
 * const onClickFunction = new Function("state", "return " + json.onClick.value)(componentState);
 *
 * // 在组件中定义一个函数来执行这个方法
 * function handleOnClick() {
 *   onClickFunction.call(this);
 * }
 *
 * // 渲染组件时，将这个函数绑定到相应的事件上
 * <button onClick={handleOnClick}>Click me</button>
 */